// perlin.cpp

#include "main.h"
#include "utility.h"

float noise1(int n); // -1 to 1
float smooth_noise1(float x);
float interpolated_noise1(float x);
float perlin_noise1(float x, float y);
float noise2(int x, int y); // -1 to 1
float smooth_noise2(float x, float y);
float interpolated_noise2(float x, float y);
float perlin_noise2(float x, float y, float persistence=.2, int noctave=4);
float cubic_interp(float v0, float v1, float v2, float v3, float x);
void  perlin_do(float persistence, int noctave);

//-------------------------------------------------------------------------

/*
  A marbly texture can be made by using a Perlin function as an offset to a cosine function.
  texture = cosine( x + perlin(x,y,z) )

  Very nice wood textures can be defined. The grain is defined with a low persistence function like this: 
  g = perlin(x,y,z) * 20
  grain = g - int(g)
  The very fine bumps yo
  u can see on the wood are high frequency noise that has been stretched in one dimension. 
  bumps = perlin(x*50, y*50, z*20)
  if bumps < .5 then bumps = 0  else bumps = 1t
*/

//-------------------------------------------------------------------------

float noise1(int n) { // -1 to 1
	n=(n<<13)^n;
	return (1-((n*(n*n*15731+789221)+1376312589)&0x7fffffff)/1073741824.0);
}

//-------------------------------------------------------------------------

float smooth_noise1(int x) {
	return noise1(x)/2 + noise1(x-1)/4 + noise1(x+1)/4;
}

//--------------------------------------------------------------------------

float interpolated_noise1(float x) {
	int integer_X;
	float fractional_X,v1,v2;

	integer_X = int(x);
	fractional_X = x - integer_X;
	v1 = smooth_noise1(integer_X);
	v2 = smooth_noise1(integer_X + 1);
	return cos_interp(v1, v2, fractional_X);
}

//--------------------------------------------------------------------------

float perlin_noise1(float x, float y) {
	float total, freq, ampl,n, persistence=.25;
	int i, noctave=6;

	total = 0;
	n = noctave - 1;
	for (i=0; i<=n; i++) {
		freq = pow(2,i);
		ampl = pow(persistence,i);
		total=total+interpolated_noise1(x*freq)*ampl;
	}
	return total;
}

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------

float noise2(int x, int y) { // -1 to 1
	int n;

	n= x+y*57;
	n=(n<<13)^n;
	return (1-((n*(n*n*15731+789221)+1376312589)&0x7fffffff)/1073741824.0);
}

//-------------------------------------------------------------------------


float smooth_noise2(float x, float y) {
	float corners, sides, center;
	corners=(noise2(x-1,y-1)+
		      noise2(x+1,y-1)+
				noise2(x-1,y+1)+
				noise2(x+1,y+1))/16;
	sides  =(noise2(x-1, y)+
		      noise2(x+1, y)+
				noise2(x, y-1)+
				noise2(x, y+1))/ 8;
	center = noise2(x, y)/4;
	return corners + sides + center;
}

//--------------------------------------------------------------------------

float interpolated_noise2(float x, float y) {
	int integer_X, integer_Y;
	float fractional_X, fractional_Y, v1,v2,v3,v4,i1,i2;

	integer_X=int(x);
	integer_Y=int(y);
	fractional_X=x-integer_X;
	fractional_Y=y-integer_Y;

	v1=smooth_noise2(integer_X,    integer_Y);
	v2=smooth_noise2(integer_X + 1,integer_Y);
	v3=smooth_noise2(integer_X,    integer_Y+1);
	v4=smooth_noise2(integer_X + 1,integer_Y+1);
	i1=cos_interp(v1,v2,fractional_X);
	i2=cos_interp(v3,v4,fractional_X);
	return cos_interp(i1,i2,fractional_Y);
}

//----------------------------------------------------------------------------

float perlin_noise2(float x, float y, float persistence, int noctave) {
	float total, freq, ampl;
	int i;

	total = 0;
	for (i=0; i<=noctave-1; i++) {
		freq = pow(2,i);
		ampl = pow(persistence,i);
		total=total+interpolated_noise2(x*freq,y*freq)*ampl;
	}
	return total;
}

//--------------------------------------------------------------------------
/*
v0 = the point before a 
v1 = the point a 
v2 = the point b 
v3 = the point after b 
*/
//-------------------------------------------------------------------------

float cubic_interp(float v0, float v1, float v2, float v3, float x) {
	float p,q,r,s;

	p = (v3 - v2) - (v0 - v1);
   q = (v0 - v1) - p;
   r =  v2 - v0;
   s =  v1;
	return p*pow(x,3)+q*pow(x,2)+r*x+s;
}

//-----------------------------------------------------------------------

void perlin_do(float persistence, int noctave) {
	int x,y, lum;
	unsigned char array[128][128][4];

	for    (x=0; x<128; x++) {
		for (y=0; y<128; y++) {
			lum=(perlin_noise2(x,y, persistence, noctave)+1)*127;
			array[x][y][0]=lum;
			array[x][y][1]=lum;
			array[x][y][2]=lum;
			array[x][y][3]=lum;
		}
	}
	//glBindTexture(GL_TEXTURE_2D, tex_perlin);
	glTexSubImage2D(GL_TEXTURE_2D,0, 0,0,128,128, GL_RGBA,GL_UNSIGNED_BYTE,array);
	return;
}

//----------------------------------------------------------------------------


